@@ -10,7 +10,7 @@ use syn::{
 };
 
 use super::{kw, metadata::Metadata};
-use crate::util::{all_cfgs, all_cfgs_expr, extract_cfg};
+use crate::util::{all_cfgs, extract_cfg};
 
 /// The result of processing the `request` section of the macro.
 pub(crate) struct Request {
@@ -81,16 +81,10 @@ impl Request {
         let struct_attributes = &self.attributes;
 
         let method = &metadata.method;
-        let path = &metadata.path;
-        let auth_attributes = metadata.authentication.iter().map(|field| {
-            let cfg_expr = all_cfgs_expr(&field.attrs);
-            let value = &field.value;
-
-            match cfg_expr {
-                Some(expr) => quote! { #[cfg_attr(#expr, ruma_api(authentication = #value))] },
-                None => quote! { #[ruma_api(authentication = #value)] },
-            }
-        });
+        let authentication = &metadata.authentication;
+        let unstable_attr = metadata.unstable_path.as_ref().map(|p| quote! { unstable = #p, });
+        let r0_attr = metadata.r0_path.as_ref().map(|p| quote! { r0 = #p, });
+        let stable_attr = metadata.stable_path.as_ref().map(|p| quote! { stable = #p, });
 
         let request_ident = Ident::new("Request", self.request_kw.span());
         let lifetimes = self.all_lifetimes();
@@ -110,10 +104,12 @@ impl Request {
             #[incoming_derive(!Deserialize, #ruma_api_macros::_FakeDeriveRumaApi)]
             #[ruma_api(
                 method = #method,
-                path = #path,
+                authentication = #authentication,
+                #unstable_attr
+                #r0_attr
+                #stable_attr
                 error_ty = #error_ty,
             )]
-            #( #auth_attributes )*
             #( #struct_attributes )*
             pub struct #request_ident < #(#lifetimes),* > {
                 #fields
