@@ -1,10 +1,9 @@
-use proc_macro2::{Ident, Span, TokenStream};
+use proc_macro2::TokenStream;
 use quote::quote;
-use syn::Field;
-
-use crate::auth_scheme::AuthScheme;
+use syn::{Field, LitStr};
 
 use super::{Request, RequestField};
+use crate::{auth_scheme::AuthScheme, util};
 
 impl Request {
     pub fn expand_outgoing(&self, ruma_api: &TokenStream) -> TokenStream {
@@ -15,39 +14,29 @@ impl Request {
 
         let method = &self.method;
         let error_ty = &self.error_ty;
-        let request_path_string = if self.has_path_fields() {
-            let mut format_string = self.path.value();
-            let mut format_args = Vec::new();
 
-            while let Some(start_of_segment) = format_string.find(':') {
-                // ':' should only ever appear at the start of a segment
-                assert_eq!(&format_string[start_of_segment - 1..start_of_segment], "/");
+        let (unstable_path, r0_path, stable_path) = if self.has_path_fields() {
+            let path_format_args_call_with_percent_encoding = |s: &LitStr| -> TokenStream {
+                util::path_format_args_call(s.value(), &percent_encoding)
+            };
 
-                let end_of_segment = match format_string[start_of_segment..].find('/') {
-                    Some(rel_pos) => start_of_segment + rel_pos,
-                    None => format_string.len(),
-                };
-
-                let path_var = Ident::new(
-                    &format_string[start_of_segment + 1..end_of_segment],
-                    Span::call_site(),
-                );
-                format_args.push(quote! {
-                    #percent_encoding::utf8_percent_encode(
-                        &::std::string::ToString::to_string(&self.#path_var),
-                        #percent_encoding::NON_ALPHANUMERIC,
-                    )
-                });
-                format_string.replace_range(start_of_segment..end_of_segment, "{}");
-            }
-
-            quote! {
-                format_args!(#format_string, #(#format_args),*)
-            }
+            (
+                self.unstable_path.as_ref().map(path_format_args_call_with_percent_encoding),
+                self.r0_path.as_ref().map(path_format_args_call_with_percent_encoding),
+                self.stable_path.as_ref().map(path_format_args_call_with_percent_encoding),
+            )
         } else {
-            quote! { metadata.path.to_owned() }
+            (
+                self.unstable_path.as_ref().map(|path| quote! { format_args!(#path) }),
+                self.r0_path.as_ref().map(|path| quote! { format_args!(#path) }),
+                self.stable_path.as_ref().map(|path| quote! { format_args!(#path) }),
+            )
         };
 
+        let unstable_path = util::map_option_literal(&unstable_path);
+        let r0_path = util::map_option_literal(&r0_path);
+        let stable_path = util::map_option_literal(&stable_path);
+
         let request_query_string = if let Some(field) = self.query_map_field() {
             let field_name = field.ident.as_ref().expect("expected field to have identifier");
 
@@ -176,8 +165,10 @@ impl Request {
             quote! {
                 #ruma_serde::json_to_buf(&RequestBody { #initializers })?
             }
-        } else {
+        } else if method == "GET" {
             quote! { <T as ::std::default::Default>::default() }
+        } else {
+            quote! { #ruma_serde::slice_to_buf(b"{}") }
         };
 
         let (impl_generics, ty_generics, where_clause) = self.generics.split_for_impl();
@@ -204,6 +195,7 @@ impl Request {
                     self,
                     base_url: &::std::primitive::str,
                     access_token: #ruma_api::SendAccessToken<'_>,
+                    considering_versions: &'_ [#ruma_api::MatrixVersion],
                 ) -> ::std::result::Result<#http::Request<T>, #ruma_api::error::IntoHttpError> {
                     let metadata = self::METADATA;
 
@@ -212,7 +204,7 @@ impl Request {
                         .uri(::std::format!(
                             "{}{}{}",
                             base_url.strip_suffix('/').unwrap_or(base_url),
-                            #request_path_string,
+                            #ruma_api::select_path(considering_versions, &metadata, #unstable_path, #r0_path, #stable_path)?,
                             #request_query_string,
                         ));
 
